@@ -17,7 +17,14 @@
 #    under the License.
 
 import collections
+# TODO(smcginnis) update this once six has support for collections.abc
+# (https://github.com/benjaminp/six/pull/241) or clean up once we drop py2.7.
+try:
+    from collections.abc import Iterable
+except ImportError:
+    from collections import Iterable
 import contextlib
+import inspect as pyinspect
 import itertools
 import logging
 import re
@@ -180,7 +187,9 @@ def paginate_query(query, model, limit, sort_keys, marker=None,
         LOG.warning('Unique keys not in sort_keys. '
                     'The sorting order may be unstable.')
 
-    assert(not (sort_dir and sort_dirs))
+    if sort_dir and sort_dirs:
+        raise AssertionError('Disallow set sort_dir and '
+                             'sort_dirs at the same time.')
 
     # Default the sort direction to ascending
     if sort_dirs is None and sort_dir is None:
@@ -190,7 +199,8 @@ def paginate_query(query, model, limit, sort_keys, marker=None,
     if sort_dirs is None:
         sort_dirs = [sort_dir for _sort_key in sort_keys]
 
-    assert(len(sort_dirs) == len(sort_keys))
+    if len(sort_dirs) != len(sort_keys):
+        raise AssertionError('sort_dirs and sort_keys must have same length.')
 
     # Add sorting
     for current_sort_key, current_sort_dir in zip(sort_keys, sort_dirs):
@@ -276,8 +286,7 @@ def paginate_query(query, model, limit, sort_keys, marker=None,
 def to_list(x, default=None):
     if x is None:
         return default
-    if not isinstance(x, collections.Iterable) or \
-            isinstance(x, six.string_types):
+    if not isinstance(x, Iterable) or isinstance(x, six.string_types):
         return [x]
     elif isinstance(x, list):
         return x
@@ -711,54 +720,6 @@ def _change_deleted_column_type_to_id_type_sqlite(engine, table_name,
         execute()
 
 
-def get_connect_string(backend, database, user=None, passwd=None,
-                       host='localhost'):
-    """Get database connection
-
-    Try to get a connection with a very specific set of values, if we get
-    these then we'll run the tests, otherwise they are skipped
-
-    DEPRECATED: this function is deprecated and will be removed from oslo_db
-    in a few releases. Please use the provisioning system for dealing
-    with URLs and database provisioning.
-
-    """
-    args = {'backend': backend,
-            'user': user,
-            'passwd': passwd,
-            'host': host,
-            'database': database}
-    if backend == 'sqlite':
-        template = '%(backend)s:///%(database)s'
-    else:
-        template = "%(backend)s://%(user)s:%(passwd)s@%(host)s/%(database)s"
-    return template % args
-
-
-def is_backend_avail(backend, database, user=None, passwd=None):
-    """Return True if the given backend is available.
-
-
-    DEPRECATED: this function is deprecated and will be removed from oslo_db
-    in a few releases. Please use the provisioning system to access
-    databases based on backend availability.
-
-    """
-    from oslo_db.sqlalchemy import provision
-
-    connect_uri = get_connect_string(backend=backend,
-                                     database=database,
-                                     user=user,
-                                     passwd=passwd)
-    try:
-        eng = provision.Backend._ensure_backend_available(connect_uri)
-        eng.dispose()
-    except exception.BackendNotAvailable:
-        return False
-    else:
-        return True
-
-
 def get_db_connection_info(conn_pieces):
     database = conn_pieces.path.strip('/')
     loc_pieces = conn_pieces.netloc.split('@')
@@ -1199,7 +1160,8 @@ def suspend_fk_constraints_for_col_alter(
             fks = []
             for ref_table_name in referents:
                 for fk in insp.get_foreign_keys(ref_table_name):
-                    assert fk.get('name')
+                    if not fk.get('name'):
+                        raise AssertionError("foreign key hasn't a name.")
                     if fk['referred_table'] == table_name and \
                             column_name in fk['referred_columns']:
                         fk['source_table'] = ref_table_name
@@ -1227,6 +1189,25 @@ def suspend_fk_constraints_for_col_alter(
                 )
 
 
+def getargspec(fn):
+    """Inspects a function for its argspec.
+
+    This is to handle a difference between py2/3. The Python 2.x getargspec
+    call is deprecated in Python 3.x, with the suggestion to use the signature
+    call instead.
+
+    To keep compatibility with the results, while avoiding deprecation
+    warnings, this instead will use the getfullargspec instead.
+
+    :param fn: The function to inspect.
+    :returns: The argspec for the function.
+    """
+    if hasattr(pyinspect, 'getfullargspec'):
+        return pyinspect.getfullargspec(fn)
+
+    return pyinspect.getargspec(fn)
+
+
 class NonCommittingConnectable(object):
     """A ``Connectable`` substitute which rolls all operations back.
 
@@ -1246,12 +1227,16 @@ class NonCommittingConnectable(object):
 
     """
 
+    _nested_trans = None
+
     def __init__(self, connection):
         self.connection = connection
         self._trans = connection.begin()
         self._restart_nested()
 
     def _restart_nested(self):
+        if self._nested_trans is not None:
+            self._nested_trans.rollback()
         self._nested_trans = self.connection.begin_nested()
 
     def _dispose(self):
